<#
.SYNOPSIS
    Issue a call to SharePoint Online to delete all metadata from on-premises content that was
    indexed through cloud hybrid search. This operation is asynchronous.
.PARAMETER PortalUrl
    SharePoint Online portal URL, for example 'https://contoso.sharepoint.com'.
.LAST UPDATED
    2020-10-15
#>
param(
    [Parameter(Mandatory=$true, HelpMessage="SharePoint Online portal URL, for example 'https://contoso.sharepoint.com'.")]
    [ValidateNotNullOrEmpty()]
    [String] $PortalUrl
)

function Get-SPVersion {
    $ver = "15"
    $regKey = Get-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Office Server\15.0\Search" -ErrorAction SilentlyContinue
    if ($null -eq $regKey) {
        $regKey = Get-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Office Server\16.0\Search" -ErrorAction SilentlyContinue
        if ($null -eq $regKey) {
            throw "Unable to detect SharePoint installation."
        }
        $ver = "16"
    }

    return $ver;
}

function Get-ClientContext
{
    param(
        [Parameter(Mandatory=$true)]
        [string] $PortalUrl,

        [Parameter(Mandatory=$true)]
        [int] $SpVer
    )

$code = @"
using System;
using System.Net;
using System.Security;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Client;
using Microsoft.SharePoint.IdentityModel;
using Microsoft.SharePoint.IdentityModel.OAuth2;

static public class ClientContextHelper
{
    public static ClientContext GetAppClientContext(string siteUrl)
    {
        SPServiceContext serviceContext = SPServiceContext.GetContext(SPServiceApplicationProxyGroup.Default, SPSiteSubscriptionIdentifier.Default);
        using (SPServiceContextScope serviceContextScope = new SPServiceContextScope(serviceContext))
        {
            ClientContext clientContext = new ClientContext(siteUrl);
            ICredentials credentials = null;
            clientContext.ExecutingWebRequest += (sndr, request) =>
            {
                    request.WebRequestExecutor.RequestHeaders.Add(HttpRequestHeader.Authorization, "Bearer");
                    request.WebRequestExecutor.WebRequest.PreAuthenticate = true;
            };
            // Run elevated to get app credentials
            SPSecurity.RunWithElevatedPrivileges(delegate()
            {
               credentials = SPOAuth2BearerCredentials.Create();
            });
            clientContext.Credentials = credentials;
            return clientContext;
        }
    }
}
"@
    $assemblies = @(
    "System.Core.dll",
    "System.Web.dll",
    "Microsoft.SharePoint, Version=$SpVer.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c",
    "Microsoft.SharePoint.Client, Version=$SpVer.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c",
    "Microsoft.SharePoint.Client.Runtime, Version=$SpVer.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c"
    )
    Add-Type -AssemblyName ("Microsoft.SharePoint.Client, Version=$SpVer.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c")
    Add-Type -AssemblyName ("Microsoft.SharePoint.Client.Search, Version=$SpVer.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c")
    Add-Type -AssemblyName ("Microsoft.SharePoint.Client.Runtime, Version=$SpVer.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c")
    Add-Type -TypeDefinition $code -ReferencedAssemblies $assemblies

    return [ClientContextHelper]::GetAppClientContext($PortalUrl)
}

try {
    Write-Host "Please Input [Y] if you want all metadata from on-premises content that was indexed through cloud hybrid search to be deleted"
    $response = Read-Host
    if ($response -ne "Y" ) { exit }

    $SP_VERSION = Get-SPVersion
    Add-Type -AssemblyName ("Microsoft.SharePoint.Client.Search, Version=$SP_VERSION.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c")
    $context = Get-ClientContext -portalurl $PortalUrl -spver $SP_VERSION
    $manager = New-Object Microsoft.SharePoint.Client.Search.ContentPush.PushTenantManager $context
    
    $task = $manager.DeleteAllCloudHybridSearchContent()
    $context.ExecuteQuery()

    Write-Host "Started delete task (id=$($task.Value))"
}
catch {
    Write-Host 'Error:' -ForegroundColor red
    Write-Host $_.Exception.Message -ForegroundColor red
    Write-Host $_.ScriptStackTrace -ForegroundColor red
}
